@@ -52,9 +52,9 @@ class GenericForeignKey(object):
         # using this model
         ContentType = get_model("contenttypes", "contenttype")
         if obj:
-             return ContentType.objects.db_manager(obj._state.db).get_for_model(obj)
+            return ContentType.objects.db_manager(obj._state.db).get_for_model(obj)
         elif id:
-             return ContentType.objects.db_manager(using).get_for_id(id)
+            return ContentType.objects.db_manager(using).get_for_id(id)
         else:
             # This should never happen. I love comments like this, don't you?
             raise Exception("Impossible arguments to GFK.get_content_type!")
@@ -339,13 +339,12 @@ class BaseGenericInlineFormSet(BaseModelFormSet):
             prefix=prefix
         )
 
-    #@classmethod
+    @classmethod
     def get_default_prefix(cls):
         opts = cls.model._meta
         return '-'.join((opts.app_label, opts.object_name.lower(),
                         cls.ct_field.name, cls.ct_fk_field.name,
         ))
-    get_default_prefix = classmethod(get_default_prefix)
 
     def save_new(self, form, commit=True):
         # Avoid a circular import.
@@ -363,7 +362,7 @@ def generic_inlineformset_factory(model, form=ModelForm,
                                   fields=None, exclude=None,
                                   extra=3, can_order=False, can_delete=True,
                                   max_num=None,
-                                  formfield_callback=lambda f: f.formfield()):
+                                  formfield_callback=None):
     """
     Returns an ``GenericInlineFormSet`` for the given kwargs.
 
@@ -397,7 +396,7 @@ class GenericInlineModelAdmin(InlineModelAdmin):
     ct_fk_field = "object_id"
     formset = BaseGenericInlineFormSet
 
-    def get_formset(self, request, obj=None):
+    def get_formset(self, request, obj=None, **kwargs):
         if self.declared_fieldsets:
             fields = flatten_fieldsets(self.declared_fieldsets)
         else:
@@ -421,6 +420,7 @@ class GenericInlineModelAdmin(InlineModelAdmin):
             "max_num": self.max_num,
             "exclude": exclude
         }
+        defaults.update(kwargs)
         return generic_inlineformset_factory(self.model, **defaults)
 
 class GenericStackedInline(GenericInlineModelAdmin):
