@@ -3,11 +3,8 @@ The main QuerySet implementation. This provides the public API for the ORM.
 """
 
 import copy
-from itertools import izip
 
 from django.db import connections, router, transaction, IntegrityError
-from django.db.models.aggregates import Aggregate
-from django.db.models.fields import DateField
 from django.db.models.query_utils import (Q, select_related_descend,
     deferred_class_factory, InvalidQuery)
 from django.db.models.deletion import Collector
@@ -231,9 +228,6 @@ class QuerySet(object):
             fields = self.model._meta.fields
             pk_idx = self.model._meta.pk_index()
 
-        index_start = len(extra_select)
-        aggregate_start = index_start + len(self.model._meta.fields)
-
         load_fields = []
         # If only/defer clauses have been specified,
         # build the list of fields that are to be loaded.
@@ -253,6 +247,9 @@ class QuerySet(object):
                     # Therefore, we need to load all fields from this model
                     load_fields.append(field.name)
 
+        index_start = len(extra_select)
+        aggregate_start = index_start + len(load_fields or self.model._meta.fields)
+
         skip = None
         if load_fields and not fill_cache:
             # Some fields have been deferred, so we have to initialise
@@ -403,6 +400,7 @@ class QuerySet(object):
                 "Cannot change a query once a slice has been taken."
         obj = self._clone()
         obj.query.set_limits(high=1)
+        obj.query.clear_ordering()
         obj.query.add_ordering('-%s' % latest_by)
         return obj.get()
 
@@ -435,6 +433,7 @@ class QuerySet(object):
         del_query._for_write = True
 
         # Disable non-supported fields.
+        del_query.query.select_for_update = False
         del_query.query.select_related = False
         del_query.query.clear_ordering()
 
@@ -583,6 +582,18 @@ class QuerySet(object):
         else:
             return self._filter_or_exclude(None, **filter_obj)
 
+    def select_for_update(self, **kwargs):
+        """
+        Returns a new QuerySet instance that will select objects with a
+        FOR UPDATE lock.
+        """
+        # Default to false for nowait
+        nowait = kwargs.pop('nowait', False)
+        obj = self._clone()
+        obj.query.select_for_update = True
+        obj.query.select_for_update_nowait = nowait
+        return obj
+
     def select_related(self, *fields, **kwargs):
         """
         Returns a new QuerySet instance that will select related objects.
@@ -712,7 +723,7 @@ class QuerySet(object):
 
     def using(self, alias):
         """
-        Selects which database this QuerySet should excecute it's query against.
+        Selects which database this QuerySet should excecute its query against.
         """
         clone = self._clone()
         clone._db = alias
@@ -1118,6 +1129,14 @@ class EmptyQuerySet(QuerySet):
         """
         return 0
 
+    def aggregate(self, *args, **kwargs):
+        """
+        Return a dict mapping the aggregate names to None
+        """
+        for arg in args:
+            kwargs[arg.default_alias] = arg
+        return dict([(key, None) for key in kwargs])
+
     # EmptyQuerySet is always an empty result in where-clauses (and similar
     # situations).
     value_annotation = False
@@ -1371,7 +1390,7 @@ class RawQuerySet(object):
             yield instance
 
     def __repr__(self):
-        return "<RawQuerySet: %r>" % (self.raw_query % self.params)
+        return "<RawQuerySet: %r>" % (self.raw_query % tuple(self.params))
 
     def __getitem__(self, k):
         return list(self)[k]
