@@ -7,7 +7,6 @@ from datetime import datetime
 from django.contrib import admin, messages
 from django.contrib.auth.decorators import permission_required
 from django.conf import settings as django_settings
-from django.core.urlresolvers import get_callable
 from django.db import models
 from django.template.defaultfilters import filesizeformat
 from django.utils.safestring import mark_safe
@@ -19,6 +18,7 @@ from django.http import HttpResponseRedirect
 
 from feincms import settings
 from feincms.models import Base
+from feincms.utils import get_object
 
 from feincms.templatetags import feincms_thumbnail
 from feincms.translations import TranslatedObjectMixin, Translation, \
@@ -40,7 +40,10 @@ class CategoryManager(models.Manager):
 
 # ------------------------------------------------------------------------
 class Category(models.Model):
-    objects = CategoryManager()
+    """
+    These categories are meant primarily for organizing media files in the
+    library.
+    """
 
     title = models.CharField(_('title'), max_length=200)
     parent = models.ForeignKey('self', blank=True, null=True,
@@ -54,6 +57,8 @@ class Category(models.Model):
         verbose_name = _('category')
         verbose_name_plural = _('categories')
 
+    objects = CategoryManager()
+
     def __unicode__(self):
         if self.parent_id:
             return u'%s - %s' % (self.parent.title, self.title)
@@ -77,11 +82,15 @@ class CategoryAdmin(admin.ModelAdmin):
 
 # ------------------------------------------------------------------------
 class MediaFileBase(Base, TranslatedObjectMixin):
+    """
+    Abstract media file class. Inherits from :class:`feincms.module.Base`
+    because of the (handy) extension mechanism.
+    """
 
     from django.core.files.storage import FileSystemStorage
     default_storage_class = getattr(django_settings, 'DEFAULT_FILE_STORAGE',
                                     'django.core.files.storage.FileSystemStorage')
-    default_storage = get_callable(default_storage_class)
+    default_storage = get_object(default_storage_class)
 
     fs = default_storage(location=settings.FEINCMS_MEDIALIBRARY_ROOT,
                            base_url=settings.FEINCMS_MEDIALIBRARY_URL)
@@ -106,11 +115,6 @@ class MediaFileBase(Base, TranslatedObjectMixin):
     filetypes = [ ]
     filetypes_dict = { }
 
-    def get_categories_as_string(self):
-        categories_tmp = self.categories.values_list('title', flat=True)
-        return ', '.join(categories_tmp)
-    get_categories_as_string.short_description = _('categories')
-
     def formatted_file_size(self):
         return filesizeformat(self.file_size)
     formatted_file_size.short_description = _("file size")
@@ -294,11 +298,14 @@ class MediaFile(MediaFileBase):
     @classmethod
     def register_extension(cls, register_fn):
         register_fn(cls, MediaFileAdmin)
-        pass
 
 # ------------------------------------------------------------------------
 # ------------------------------------------------------------------------
 class MediaFileTranslation(Translation(MediaFile)):
+    """
+    Translated media file caption and description.
+    """
+
     caption = models.CharField(_('caption'), max_length=200)
     description = models.TextField(_('description'), blank=True)
 
@@ -406,8 +413,6 @@ class MediaFileAdmin(admin.ModelAdmin):
                 messages.error(request, _("ZIP file invalid: %s") % str(e))
                 return
 
-            pass
-
         if request.method == 'POST' and 'data' in request.FILES:
             import_zipfile(request, request.POST.get('category'), request.FILES['data'])
         else:
